import org.antlr.v4.runtime.misc.NotNull;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class EvalVisitas extends GramaticaBaseVisitor<Evaluador> {

    // used to compare floating point numbers
    

    // store variables (there's only one global scope!)
    private Map<String, Evaluador> memory = new HashMap<String, Evaluador>();

    // assignment/id overrides
    @Override
    public Evaluador visitAsignacion(GramaticaParser.AsignacionContext ctx) {
        String id = ctx.ID().getText();
        Evaluador value = this.visit(ctx.expr());
        if ( memory.containsKey(id))
							{ 
								return memory.put(id, value);
							}
							else
							{
								 System.out.println("No esta declarada la variable: "+id);
								System.exit(0);
							 }
        return memory.put(id, value);
    }

    @Override
    public Evaluador visitId(GramaticaParser.IdContext ctx) {
        String id = ctx.getText();
        Evaluador value = memory.get(id);
        if(value == null) {
            throw new RuntimeException("no such variable: " + id);
        }
        return value;
    }


    @Override
    public Evaluador visitInt(GramaticaParser.IntContext ctx) {
        return new Evaluador(Double.valueOf(ctx.getText()));
    }

    @Override
    public Evaluador visitParentesiss(GramaticaParser.ParentesissContext ctx) {
        return this.visit(ctx.expr());
    }


    @Override
    public Evaluador visitMulDiv(@NotNull GramaticaParser.MulDivContext ctx) {

       Evaluador primertermino = this.visit( ctx.expr(0) );
		Evaluador segundotermino = this.visit( ctx.expr(1) );
		if(ctx.operacion.getType() == GramaticaParser.MULTI)
			return new Evaluador(primertermino.convertDouble() * segundotermino.convertDouble());

		return new Evaluador(primertermino.convertDouble() / segundotermino.convertDouble());
    }

    @Override
    public Evaluador visitAddSub(@NotNull GramaticaParser.AddSubContext ctx) {

       Evaluador primertermino = this.visit( ctx.expr(0) );
		Evaluador segundotermino = this.visit( ctx.expr(1) );
		if(ctx.operacion.getType() == GramaticaParser.SUMA)
			return new Evaluador(primertermino.convertDouble() + segundotermino.convertDouble());

		return new Evaluador(primertermino.convertDouble() - segundotermino.convertDouble());
    }

    @Override
    public Evaluador visitRelationalExpr(@NotNull GramaticaParser.RelationalExprContext ctx) {

        Evaluador left = this.visit(ctx.expr(0));
        Evaluador right = this.visit(ctx.expr(1));

        switch (ctx.operacion.getType()) {
            case GramaticaParser.MENOR:
                return new Evaluador(left.convertDouble() < right.convertDouble());
            case GramaticaParser.MENOR_IGUAL:
                return new Evaluador(left.convertDouble() <= right.convertDouble());
            case GramaticaParser.MAYOR:
                return new Evaluador(left.convertDouble() > right.convertDouble());
            case GramaticaParser.MAYOR_IGUAL:
                return new Evaluador(left.convertDouble() >= right.convertDouble());
            default:
                throw new RuntimeException("unknown operator: " + GramaticaParser.tokenNames[ctx.operacion.getType()]);
        }
    }



    // log override
    @Override
    public Evaluador visitImpresion(GramaticaParser.ImpresionContext ctx) {
        Evaluador value = this.visit(ctx.expr());
        System.out.println(value);
        return value;
    }

	@Override public Evaluador visitCondicion_if(GramaticaParser.Condicion_ifContext ctx) 
    { 
                List<GramaticaParser.Bloque_condicionContext> conditions =  ctx.bloque_condicion();
        boolean evaluatedBlock = false;
         for(GramaticaParser.Bloque_condicionContext condition : conditions) {
         
         Evaluador evaluated = this.visit(condition.expr());
         if(evaluated.convertBoolean()) {
             evaluatedBlock = true;
             this.visit(condition.bloque_llaves());
                break;
         }

         }
         if(!evaluatedBlock && ctx.bloque_llaves() != null) {
            this.visit(ctx.bloque_llaves());
         }
    return Evaluador.VOID;

    }

    // if override
   /* @Override
    public Evaluador Condicion_if(GramaticaParser.Condicion_ifContext ctx) {

        List<GramaticaParser.Bloque_condicionContext> conditions =  ctx.bloque_condicion();

        boolean evaluatedBlock = false;

        for(GramaticaParser.Bloque_condicionContext condition : conditions) {

            Evaluador evaluated = this.visit(condition.expr());

            if(evaluated.convertBoolean()) {
                evaluatedBlock = true;
                // evaluate this block whose expr==true
                this.visit(condition.bloque_llaves());
                break;
            }
        }

        if(!evaluatedBlock && ctx.bloque_llaves() != null) {
            // evaluate the else-stat_block (if present == not null)
            this.visit(ctx.bloque_llaves());
        }

        return Evaluador.VOID;
    }
*/
    // while override
    @Override
    public Evaluador visitCiclo_while(GramaticaParser.Ciclo_whileContext ctx) {

        Evaluador value = this.visit(ctx.expr());

        while(value.convertBoolean()) {

            // evaluate the code block
            this.visit(ctx.bloque_llaves());

            // evaluate the expression
            value = this.visit(ctx.expr());
        }

        return Evaluador.VOID;
    }

    	@Override public Evaluador visitDeclaracion(GramaticaParser.DeclaracionContext ctx) 
        {
        String id = ctx.ID().getText();
        Evaluador value = null;
        memory.put(id,value);
        System.out.println(memory);
        return memory.put(id,value);
        }

}